/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	chdir.c - change directory routines
 *      -----------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:02:56 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"				/* local includes */

static char *cd1(char *string);
static char *wilddir(char *path, char *wildname);
static char *makenewpath(char *np);

/*---------------------------------------------------------------------------*
 *	change directory, string may contain wildcards
 *----------------------------------------------------------------------------*/
int
cd(char *string)
{
	int retval;
	char *p;
	char *newp;
	static char fcd_path[MAXPATHLEN+256] = "";
	
	if( (*string == '-') && (fcd_path[0] != '\0') &&
	    (*(string+1) == '\0' || *(string+1) == '\t' || *(string+1) == ' ')
	  )
	{
		char tmp_path[MAXPATHLEN+256];

		strcpy(tmp_path, fcd_path);
		strcpy(fcd_path, lcur_path);
		strcpy(lcur_path, tmp_path);
		return(chdir(lcur_path));		
	}
	
	if (opt_followsdir)
	{
		newp = makenewpath(string);	/* new path to try */

		if (chdir(newp) == 0)		/* try out unresolved first */
		{
			strcpy(fcd_path, lcur_path);	/* update fast cd buf */
			strcpy(lcur_path, newp);	/* update symlink path */
			return (0);
		}

		if ((p = cd1(newp)) == NULL)	/* try to resolve */
			return (-1);

		if ((retval = chdir(p)) == 0)	/* try to change dir */
		{
			strcpy(fcd_path, lcur_path);	/* update fast cd buf */
			strcpy(lcur_path, p);		/* update symlink path */
			return (0);
		}
		return (retval);
	}
	else
	{
		if (chdir(string) == 0)		/* try out unresolved first */
		{
			strcpy(fcd_path, lcur_path);		/* update fast cd buf */
			strcpy(lcur_path, makenewpath(string));	/* update symlink path */
			return (0);
		}

		if ((p = cd1(string)) == NULL)	/* try to resolve */
			return (-1);

		if ((retval = chdir(p)) == 0)	/* try to change dir */
		{
			strcpy(fcd_path, lcur_path);	   /* update fast cd buf */
			strcpy(lcur_path, makenewpath(p)); /* update symlink path */
			return (0);
		}
		return (retval);
	}
}

/*----------------------------------------------------------------------------*
 *	make new path from lcur_path and argument
 *----------------------------------------------------------------------------*/
static char *
makenewpath(char *new_path)
{
	register char *p;
	register char *r;
	register char *bp;

	char buffer[MAXPATHLEN + 256];		/* working buffer */
	char npath[MAXPATHLEN + 256];		/* "normalized" new path */
	static char result[MAXPATHLEN + 256];	/* result buffer */

	/* "normalize" the target path */

	p = new_path;				/* source */
	r = npath;				/* dest */

	/* remove multiple slash's in path */

	while (*p)
	{
		if ((*p == '/') && (*(p + 1) == '/'))
		{
			p++;
			continue;
		}
		*r++ = *p++;
	}

	/* remove trailing slash in path; if root, leave alone */

	if (((r - 1) > npath) && (*(r - 1) == '/'))
		r--;

	/* terminate path */

	*r = '\0';

	/* now we have a clean path to cd to */

	if (npath[0] == '/')			/* easy case, change to absolute path */
	{
		strcpy(result, npath);		/* absolute path -> result */
	}
	else
	{
		if((p = index(npath, '~')) != NULL)	/* have a ~ in path */
		{
			strcpy(result, envhome);	/* get home dir */
			p++;				/* after ~ */
		}
		else
		{
			strcpy(result, lcur_path);	/* copy current into result */
			p = npath;			/* ptr to new path */
		}

		r = result + strlen(result);	/* end of current path */

		while (*p)
		{
			bp = buffer;		/* ptr to tmp buffer */

			/* get next (left) new segment -> buffer */

			if (!strcmp(p, "/"))	/* trailing '/' in new path */
				break;		/* exit */

			if (*p == '/')		/* from last time through */
				p++;

			while (*p && (*p != '/'))	/* until segment '/' end */
				*bp++ = *p++;	/* segment -> buffer */
			*bp = '\0';		/* delimit buffer */

			/* check easy cases, first '.' */

			if (!strcmp(buffer, "."))	/* no change */
				continue;

			/* check easy cases, then '..' */

			else if (!strcmp(buffer, ".."))		/* step up */
			{
				/* search segment border form right to left */

				while ((r > result) && (*r != '/'))
					r--;

				/* delete the last (rightmost) segment */

				if (r == result)
					*(r + 1) = '\0';	/* preserve root '/' */
				else
					*r = '\0';	/* not at root yet */
			}

			/* this segment is a name */

			else
			{
				if (*(r - 1) != '/')
					strcat(r, "/");		/* cat '/' to current path */
				strcat(r, buffer);	/* cat name to current path */
				r = r + strlen(buffer);		/* advance ptr to new end */
			}
		}
	}
	return (result);
}

/*---------------------------------------------------------------------------*
 *	return resolved path or NULL if failed 
 *----------------------------------------------------------------------------*/
static char *
cd1(char *string)
{
	static char left[MAXPATHLEN + 1];	/* contains no wildcards */
	char wname[MAXPATHLEN + 1];		/* current wildname to try */

	register char *sp;			/* ptr into string */
	register char *lp;			/* ptr into left */
	register char *wp;			/* ptr into wname */
	register char *gp;			/* gp ptr */

	left[0] = wname[0] = '\0';

	sp = string;
	lp = left;

	for (;;)
	{
		wp = wname;

		if (*sp == '\0')
			return (left);

		if ((*sp == '~') && (*(sp + 1) == '/'))
		{
			lp = left;		/* $HOME -> left */
			gp = envhome;		/* $HOME ptr */
			while (*gp)		/* copy $HOME -> left */
				*lp++ = *gp++;	/* new left */
			*lp++ = '/';		/* terminate left */
			sp += 2;		/* advance sp */
			continue;		/* next segment */
		}

		while (*sp && (*sp == '/'))	/* check for '/'s */
			*lp++ = *sp++;

		while (*sp && (*sp != '/'))	/* segment between '/'s -> wname */
			*wp++ = *sp++;

		*lp = *wp = '\0';		/* terminate strings */

		if (strpbrk(wname, WILDCHARS) == NULL)	/* check for wildcards */
		{
			wp = wname;		/* wname start */
			while (*wp)		/* cat wname to left */
				*lp++ = *wp++;
			continue;		/* next segment */
		}

		if ((gp = wilddir(left, wname)) == NULL)	/* resolve wildcards */
			return (NULL);		/* not found */

		lp = left;			/* resolved -> left */
		while (*gp)
			*lp++ = *gp++;		/* new left */
		continue;			/* next segment */
	}
}

/*---------------------------------------------------------------------------*
 *	check if "wildname" matches any directory name in directory "path"
 *	return concatenated "path" and matched name from dir "
 *----------------------------------------------------------------------------*/
static char *
wilddir(char *path, char *wildname)
{
	register struct dirent *dp;		/* dir access */
	register DIR *dirp;			/* dir access */
	struct stat fdbuffer;			/* stat buffer */
	static char wtry[MAXPATHLEN + 1];	/* try buffer */
	char rpath[MAXPATHLEN + 1];		/* resulting path */

	strcpy(rpath, path);			/* path -> result path */
	strcat(rpath, ".");			/* for readdir */

	if ((dirp = opendir(rpath)) == NULL)
		return (NULL);			/* cannot open directory */

	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
	{
		if (wildmat(dp->d_name, wildname) == TRUE)	/* hit ? */
		{
			strcpy(wtry, path);	/* pathname */
			strcat(wtry, dp->d_name);	/* matchname */
			if (stat(wtry, &fdbuffer) < 0)
				continue;	/* not accessible */
			if (!(S_ISDIR(fdbuffer.st_mode)))	/* directory ? */
				continue;	/* no */
			closedir(dirp);		/* close */
			return (wtry);		/* return result */
		}
	}
	closedir(dirp);				/* close */
	return (NULL);				/* failed */
}


/*  $Revision: 1.4 $
   **
   **  Do shell-style pattern matching for ?, \, [], and * characters.
   **  Might not be robust in face of malformed patterns; e.g., "foo[a-"
   **  could cause a segmentation violation.  It is 8bit clean.
   **
   **  Written by Rich $alz, mirror!rs, Wed Nov 26 19:03:17 EST 1986.
   **  Rich $alz is now <rsalz@bbn.com>.
   **  April, 1991:  Replaced mutually-recursive calls with in-line code
   **  for the star character.
   **
   **  Special thanks to Lars Mathiesen <thorinn@diku.dk> for the ABORT code.
   **  This can greatly speed up failing wildcard patterns.  For example:
   **   pattern: -*-*-*-*-*-*-12-*-*-*-m-*-*-*
   **   text 1:  -adobe-courier-bold-o-normal--12-120-75-75-m-70-iso8859-1
   **   text 2:  -adobe-courier-bold-o-normal--12-120-75-75-X-70-iso8859-1
   **  Text 1 matches with 51 calls, while text 2 fails with 54 calls.  Without
   **  the ABORT, then it takes 22310 calls to fail.  Ugh.  The following
   **  explanation is from Lars:
   **  The precondition that must be fulfilled is that DoMatch will consume
   **  at least one character in text.  This is true if *p is neither '*' nor
   **  '\0'.)  The last return has ABORT instead of FALSE to avoid quadratic
   **  behaviour in cases like pattern "*a*b*c*d" with text "abcxxxxx".  With
   **  FALSE, each star-loop has to run to the end of the text; with ABORT
   **  only the last one does.
   **
   **  Once the control of one instance of DoMatch enters the star-loop, that
   **  instance will return either TRUE or ABORT, and any calling instance
   **  will therefore return immediately after (without calling recursively
   **  again).  In effect, only one star-loop is ever active.  It would be
   **  possible to modify the code to maintain this context explicitly,
   **  eliminating all recursive calls at the cost of some complication and
   **  loss of clarity (and the ABORT stuff seems to be unclear enough by
   **  itself).  I think it would be unwise to try to get this into a
   **  released version unless you have a good test data base to try it out
   **  on.
 */

    /* What character marks an inverted character class? */
#define NEGATE_CLASS		'!'		/* -hm , was '^' */
    /* Is "*" a common pattern? */
#define OPTIMIZE_JUST_STAR
    /* Do tar(1) matching rules, which ignore a trailing slash? */
#undef MATCH_TAR_PATTERN


/*
   **  Match text and p, return TRUE, FALSE, or ABORT.
 */
static int
DoMatch(register char *text, register char *p)
{
	register int dolast;			/* -hm, was "last" */
	register int matched;
	register int reverse;

	for (; *p; text++, p++)
	{
		if (*text == '\0' && *p != '*')
			return ABORT;
		switch (*p)
		{
			case '\\':
				/* Literal match with following character. */
				p++;
				/* FALLTHROUGH */
			default:
				if (*text != *p)
					return FALSE;
				continue;
			case '?':
				/* Match anything. */
				continue;
			case '*':
				while (*++p == '*')
					/* Consecutive stars act just like one. */
					continue;
				if (*p == '\0')
					/* Trailing star matches everything. */
					return TRUE;
				while (*text)
					if ((matched = DoMatch(text++, p)) != FALSE)
						return matched;
				return ABORT;
			case '[':
				reverse = p[1] == NEGATE_CLASS ? TRUE : FALSE;
				if (reverse)
					/* Inverted character class. */
					p++;
				for (dolast = 0400, matched = FALSE; *++p && *p != ']'; dolast = *p)
					/* This next line requires a good C compiler. */
					if (*p == '-' ? *text <= *++p && *text >= dolast : *text == *p)
						matched = TRUE;
				if (matched == reverse)
					return FALSE;
				continue;
		}
	}

#ifdef	MATCH_TAR_PATTERN
	if (*text == '/')
		return TRUE;
#endif /* MATCH_TAR_ATTERN */
	return *text == '\0';
}


/*
   **  User-level routine.  Returns TRUE or FALSE.
 */
int
wildmat(char *text, char *p)
{
#ifdef	OPTIMIZE_JUST_STAR
	if (p[0] == '*' && p[1] == '\0')
		return TRUE;
#endif /* OPTIMIZE_JUST_STAR */
	return DoMatch(text, p) == TRUE;
}

/*---------------------------------- EOF -------------------------------------*/
