/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	commandline.c - commandline handling
 *      ------------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:03:00 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"				/* local includes */


static int waitcr = 0;				/* do not wait for cr */
static int reread = 0;				/* reread current dir */
static int nochng = 0;				/* nothing has changed */
static int preserve = PRES_NO;			/* preserve highlighted position */

static char *adjustfn(register char *name);
static char *str_nxtw(char *, int);
static char *tail(char *filename);

/*---------------------------------------------------------------------------*
 *      echo the current highlighted filename onto the commandline
 *---------------------------------------------------------------------------*/
void
name_echo(void)
{
	char *ptr;				/* temp ptr */

	ptr = &(cur_file->onam[1]);		/* ptr = current filename */

	/* check for delimiter */

	if ((!(cr_on_files())) && (cbuff[curcol() - 1] != SPACE))
		cmdline(SPACE);			/* if none, insert one */

	ptr = adjustfn(ptr);

	while ((*ptr) && ((cmdline(*ptr++)) == GOOD))
		;
}

/*---------------------------------------------------------------------------*
 *      returns a pointer to the next word in string
 *---------------------------------------------------------------------------*/
static char *
str_nxtw(char *str, int frst)
{
	char *r;

	if ((str == NULL) || (*str == '\0'))
		return (NULL);

	r = str;

	if (frst)
	{
		while (*r == SPACE)		/* while leading delimiters */
			r++;
		if (*r != '\0')
			return (r);		/* first non-space char */
		else
			return (NULL);
	}
	else
	{
		while ((*r != SPACE) && (*r != '\0'))	/* find space */
			r++;
		if (*r == '\0')			/* test for end of string */
			return (NULL);
		while (*r == SPACE)		/* while there are delimiters */
			r++;
		if (*r != '\0')
			return (r);		/* first non-space char */
		else
			return (NULL);		/* end of string */
	}
}

/*---------------------------------------------------------------------------*
 *      expand the selection character onto the commandline
 *---------------------------------------------------------------------------*/
void
expsel(char *p)
{
	char rest[512];				/* save buffer */
	struct onefile *info;			/* our file struct ptr */
	int i;					/* free space counter in cbuff */
	register char *fnp;			/* for filename conversion */

	strcpy(rest, p + 1);			/* save right half of cbuff after EXPSEL-char */

	if (*(p - 1) != SPACE)			/* check for delimiter present */
	{
		*p = SPACE;
		p++;
	}

	i = (p - cbuff) + strlen(rest);		/* count of already occupied chars */

	info = first;				/* first in list */

	*p = '\0';				/* terminate left half of cbuff */

	while (info)				/* scan through dir - list */
	{
		if (info->tag)			/* search for tagged files */
		{
			fnp = adjustfn(&(info->onam[1]));

			if (((strlen(fnp)) + 1) < (HISLNLEN - i))
			{
				/* we have enough space  */

				i += (strlen(fnp) + 1);
				strcat(cbuff, fnp);
				strcat(cbuff, " ");
			}
		}
		info = info->next;		/* next entry */
	}
	strcat(cbuff, rest);			/* copy saved right half to generated part */
}

/*---------------------------------------------------------------------------*
 *      user has typed a <CR>
 *---------------------------------------------------------------------------*/
void
handlecr(void)
{
	nochng = 0;				/* anything may change on screen */
	preserve = PRES_NORM;			/* nothing to preserve */

	if (cr_on_files())			/* nothing on commandline, process high- */
	{					/*  lighted filename field */
		h_files();			/* cr on a filename */
	}
	else
		/* something in commandlinebuffer */
	{
		h_line();			/* commandline */
	}

	if (waitcr && opt_return)		/* wait for <cr> being hit */
	{
		move(LINES - 1, 0);		/* last line */
		clrtoeol();			/* clear it */
		move(LINES - 1, 0);		/* last line */
		attrset(A_REVERSE);		/* highlight on */
		addstr("Press any key to continue ...");
		attroff(A_REVERSE);		/* highlight on */
		refresh();
		getch();
		move(LINES - 1, 0);		/* last line */
		clrtoeol();			/* clear it */
		touchwin(stdscr);
		wnoutrefresh(stdscr);
	}
	header();				/* new header */
	dis_hist();				/* display current history entry */

	if (reread)				/* directory has to be re-read */
	{
		free_list();			/* free malloc'ed buffers */
		init_files(preserve, cur_path);	/* yes, read current dir */
	}
	else
	{
		if (!nochng)
			fresh_files();		/* just refresh window */
	}
	resume_time();				/* restart time display update */
	if (!nochng)
	{
		attribs(1);			/* new attributes */
		init_flabels();			/* fk-labels onto screen */
		update_all();			/* update all windows */
	}
	else
	{
		wrefresh(cmnd_w);
	}
}

/*---------------------------------------------------------------------------*
 *      user has typed <CR> and something is on commandline
 *---------------------------------------------------------------------------*/
void
h_line(void)
{
	char *cptr = cbuff + 3;			/* ptr to start of line after prompt */
	char *cp;				/* command ptr */
	char *ap;				/* argument ptr */
	char *ep;				/* expansion char ptr */

	expand_alias();				/* check if an alias needs to be expanded */

	if ((cp = str_nxtw(cptr, 1)) == NULL)	/* just spaces on commandline ? */
	{
		nochng = 1;
		errno = 0;			/* yes ... */
		error("commandline empty ... ");
		waitcr = 0;
		reread = 0;
	}
	else if ((*cp == 'c') && (*(cp + 1) == 'd') &&
	 ((*(cp + 2) == ' ') || (*(cp + 2) == '\t') || (*(cp + 2) == '\0')))
	{
		int ret;

		if ((ap = str_nxtw(cp, 0)) == NULL)
		{
			preserve = PRES_NO;	/* nothing to preserve */
			ret = cd(envhome);	/* cd to home */
		}
		else
		{
			char *ptr;

			if ((ptr = (char *) index(ap, '.')) != NULL)
			{
				if (!(strncmp(ptr, "..", 2)))
					preserve = PRES_DD;
			}
			ret = cd(ap);		/* cd to <dir> */
		}

		if (ret == -1)
		{
			nochng = 1;
			error("changing directory failed - chdir() error");
			waitcr = 0;
			reread = 0;
		}
		else
		{
			waitcr = 0;
			reread = 1;
		}
	}
	else
	{
		if ((ep = strchr((cbuff + 3), EXPSEL)) != NULL)
			expsel(ep);		/* expand selection char */

		exec_command(cbuff + 3);

		waitcr = 1;			/* wait for <cr> */
		reread = 1;			/* reread current dir */
	}
	save_line();				/* save commandline to history */
}

/*---------------------------------------------------------------------------*
 *      user has typed <CR> and commandline is empty
 *---------------------------------------------------------------------------*/
void
h_files(void)
{
	preserve = PRES_NORM;

	/* directories */

	if (cur_file->oprm[0] == 'd')
	{
		if (!strcmp(&(cur_file->onam[1]), ".."))	/* stepping up */
			preserve = PRES_DD;
		else				/* stepping down */
			preserve = PRES_NO;

		if ((cd(&(cur_file->onam[1]))) == -1)
		{
			nochng = 1;
			error("changing directory to current selection failed - chdir() error");
			reread = 0;		/* no need to re-read */
		}
		else
		{
			reread = 1;		/* (re-) read (new) dir */
		}

		waitcr = 0;			/* don't wait for <CR> */
	}

	/* symbolic link without a target */

	else if (cur_file->oslink && (cur_file->onam[0] == ISUNKN))
	{
		char lb[MAXPATHLEN + 1];
		int count;

		count = readlink(&(cur_file->onam[1]), lb, MAXPATHLEN);
		if (count == -1)
		{
			error("symbolic link without target, readlink failed!");
		}
		else
		{
			if (count > (COLS - 1))
				count = COLS - 1;
			lb[count] = '\0';	/* terminate dest */
			error(lb);
		}
		nochng = 1;
		waitcr = 0;
		reread = 0;
	}

	/* executables */

	else if ((cur_file->oprm[3] == 'x') ||
		 (cur_file->oprm[3] == 's') ||
		 (cur_file->oprm[6] == 'x') ||
		 (cur_file->oprm[6] == 's') ||
		 (cur_file->oprm[9] == 'x') ||
		 (cur_file->oprm[9] == 't'))
	{
		sprintf(cbuff, "%s/%s", cur_path, &(cur_file->onam[1]));

		exec_command(cbuff);		/* shell-call */

		waitcr = 1;
		reread = 1;
	}

	/* data files */

	else if ((cur_file->oprm[1] == 'r') ||
		 (cur_file->oprm[4] == 'r') ||
		 (cur_file->oprm[7] == 'r'))
	{
		FILE *pfp;
		char *pp;
		char *fnp;

		fnp = adjustfn(&(cur_file->onam[1]));

		sprintf(cbuff, "cat %s/%s|", cur_path, fnp);

		if ((pp = tail(fnp)) != NULL)
			strcat(cbuff, pp);

		if (cbuff[strlen(cbuff) - 1] == '|')
			strcat(cbuff, opt_more);	/* pager */

		suspend_time();			/* stop updating time */
		fini_flabels();			/* remove user labels */
		move(0, 0);			/* first line */
		touchwin(stdscr);		/* force */
		clear();			/* clear it */
		refresh();			/* update screen */
		savetty();			/* save tty-modes for resetty() */
		reset_shell_mode();		/* set tty to pre-curses values */

		signal(SIGINT,  SIG_IGN);	/* ignore INT and QUIT */
		signal(SIGQUIT, SIG_IGN);	/* signals             */
		
		if ((pfp = popen(cbuff, "w")) == NULL)	/* exec */
			error("cannot open pipe to PAGER");
		else
		{
			if (pclose(pfp) == -1)
				error("PAGER pclose error");
		}

		signal(SIGINT,  SIG_DFL);	/* restore INT and QUIT */
		signal(SIGQUIT, SIG_DFL);	/* signals              */

		resetty();			/* restore modes to curses operation */

		reset_prog_mode();		/* set to curses values */

#ifdef CURSESBUG
		putp(keypad_xmit);		/* curses bug ??!! */
#endif

		touchwin(cmnd_w);
		touchwin(fst_w);
		touchwin(file_w);
		if (opt_attrib)
			touchwin(attr_w);
		if (opt_labels)
			touchwin(flbl_w);

		resume_time();			/* restart updating time */
		waitcr = 0;			/* don't wait for <CR> */
		reread = 0;			/* reread dir */
	}

	/* OOPS, whats this ?? */
	else
	{
		errno = 0;			/* no errno display */
		error("no exec/read permission found ....");
		flash();			/* no no .. */
		waitcr = 0;			/* don't wait for <CR> */
		reread = 0;			/* don't reread dir !!! */
	}
}

/*---------------------------------------------------------------------------*
 *      what to do with the file depending on ending
 *---------------------------------------------------------------------------*/
static char *
tail(char *filename)
{
	int i;

	/* see if filename contains a known extension and return the    */
	/* commandline string to decode that file                       */

	for (i = 0; i < (sizeof(etab) / sizeof(etab[0])) && etab[i][0]; ++i)
		;

	while (i--)
	{
		if (wildmat(filename, etab[i][0]) == TRUE)
			return ((char *) (etab[i][1]));
	}
	return (NULL);
}

/*---------------------------------------------------------------------------*
 *      edit the current file
 *---------------------------------------------------------------------------*/
void
edit_current(void)
{
	char *fnp;

	if (cur_file->oprm[0] == 'd')		/* directories */
		return;

	fnp = adjustfn(&(cur_file->onam[1]));

	sprintf(cbuff, "%s %s/%s", opt_edit, cur_path, fnp);

	exec_command(cbuff);			/* shell-call */

	header();				/* new header */
	dis_hist();				/* display current history */

#ifdef REREAD_AFTER_EDIT
	free_list();				/* free malloc'ed buffers */
	init_files(PRES_NORM, NULL);		/* reread & display files */
#else
	fresh_files();				/* refresh display */
#endif

	attribs(1);				/* new attributes */
	init_flabels();				/* fk-labels onto screen */
	update_all();
	resume_time();				/* restart time display update */
}

/*---------------------------------------------------------------------------*
 *      close current directory
 *---------------------------------------------------------------------------*/
void
close_cdir(void)
{
	if ((cd("..")) == -1)
	{
		error("changing directory to '..' dir failed - chdir() error");
		return;
	}
	else
	{
		header();			/* new header */
		free_list();			/* free malloc'ed buffers */
		init_files(PRES_DD, cur_path);	/* yes, read current dir */
		attribs(1);			/* new attributes */
		init_flabels();			/* fk-labels onto screen */
	}
	update_all();
}

/*---------------------------------------------------------------------------*
 *      execute a command/program
 *---------------------------------------------------------------------------*/
void
exec_command(char *cline)
{
	suspend_time();				/* no time display update */
	fini_flabels();				/* remove user labels */
	move(0, 0);				/* last line */
	touchwin(stdscr);			/* force */
	clear();				/* clear it */
	refresh();
	savetty();				/* save tty-modes for resetty() */
	reset_shell_mode();			/* set to pre-curses values */

#ifdef USE_OWN_SYSTEM
	mysystem(cline);			/* shell-call, via own system.c  */
#else
	system(cline);				/* shell-call, system supplied */
#endif

	resetty();				/* restore modes to curses operation */
	reset_prog_mode();			/* set to curses values */

#ifdef CURSESBUG
	putp(keypad_xmit);			/* curses bug ??!! */
#endif

	touchwin(cmnd_w);
	touchwin(fst_w);
	touchwin(file_w);
	if (opt_attrib)
		touchwin(attr_w);
	if (opt_labels)
		touchwin(flbl_w);
}

/*---------------------------------------------------------------------------*
 *      adjust filename (cdrom, others ?)
 *---------------------------------------------------------------------------*/
static char *
adjustfn(char *name)
{
	static char buffer[MAXPATHLEN];
	register char *bp = buffer;
	register char *sp = name;

	while (*sp)
	{
		if (iscdfs && (*sp == ';'))
			*bp++ = '\\';
		*bp++ = *sp++;
	}
	*bp = '\0';
	return ((char *) buffer);
}

/*---------------------------------- EOF -------------------------------------*/
