/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	complete.c - filename completition
 *      ----------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:02:48 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"				/* local includes */

static char *wild(char *path, char *wildname);

/*---------------------------------------------------------------------------*
 *	complete name to first matching filename
 *----------------------------------------------------------------------------*/
void
complete(void)
{
	char *cptr;				/* ptr to start of line after prompt */
	char *cend;				/* ptr to end of commandline */
	char *p, *q;				/* gp ptr */
	char *csave;				/* filename start position */
	char path[MAXPATHLEN + 1];		/* path portion */
	char name[MAXPATHLEN + 1];		/* name portion */

	expand_tilde();				/* in history.c */
	
	cptr = cbuff + 3;			/* init start ptr */
	cend = cptr + strlen(cptr);		/* int end ptr */

	while (cend > cptr)			/* end of commandline */
	{
		cend--;
		if (isspace(*cend))		/* start of filename ? */
			break;
	}
	if (cend <= cptr)			/* hmm ... */
		return;

	cend++;					/* first char of filename */
	csave = cend;				/* save position for later compare */

	if ((p = strrchr(cend, '/')) == NULL)
	{
		path[0] = '\0';			/* setup path part */
		strcpy(name, cend);		/* setup name part */
	}
	else
	{
		q = path;			/* setup path part */
		while (cend <= p)
			*q++ = *cend++;
		*q = '\0';

		q = name;			/* setup name part */
		while (*cend)
			*q++ = *cend++;
		*q = '\0';
	}
	strcat(name, "*");			/* add wildcard char */

	if ((q = wild(path, name)) == NULL)	/* resolve wildcards */
		return;				/* not found */

	while (*csave && *q && (*csave++ == *q++))	/* skip same chars */
		;

	while (*q && (cmdline(*q++) == GOOD))	/* rest to commandline */
		;
}

/*---------------------------------------------------------------------------*
 *	check if "wildname" matches any name in directory "path"
 *	return concatenated "path" and matched name from dir
 *	if directory found, append a '/' to resulting name
 *----------------------------------------------------------------------------*/
static char *
wild(char *path, char *wildname)
{
	register struct dirent *dp;		/* dir access */
	register DIR *dirp;			/* dir access */
	struct stat fdbuffer;			/* stat buffer */
	static char wtry[MAXPATHLEN + 1];	/* try buffer */
	char rpath[MAXPATHLEN + 1];		/* resulting path */

	strcpy(rpath, path);			/* path -> result path */
	strcat(rpath, ".");			/* for readdir */

	if ((dirp = opendir(rpath)) == NULL)
		return (NULL);			/* cannot open directory */

	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
	{
		if ((!(strcmp(dp->d_name, "."))) || (!(strcmp(dp->d_name, ".."))))
			continue;

		if (wildmat(dp->d_name, wildname) == TRUE)	/* hit ? */
		{
			strcpy(wtry, path);	/* pathname */
			strcat(wtry, dp->d_name);	/* matchname */
			if (stat(wtry, &fdbuffer) < 0)
				continue;	/* not accessible */
			closedir(dirp);		/* close */
			if (S_ISDIR(fdbuffer.st_mode))	/* directory ? */
				strcat(wtry, "/");	/* yes, append / */
			return (wtry);		/* return result */
		}
	}
	closedir(dirp);				/* close */
	return (NULL);				/* failed */
}

/*---------------------------------- EOF -------------------------------------*/
