/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	config.h - runtime configuration header file
 *      --------------------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:03:12 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*
 *	config menu, structure for one item on the menu
 *---------------------------------------------------------------------------*/

struct item {
	int row;		/* name row position */
	int col;		/* name column position */
	char *name;		/* field name */
	int nl;			/* name length */
	int type;		/* item type */
#define ITEM_ONOFF	0		/* YES / NO */
#define ITEM_STRING	1		/* a string */
#define ITEM_CHAR	2		/* a character */
#define ITEM_NOCHANGE	0x80		/* user cannot chage item at run-time */
	int *value;		/* pointer to variable [...hmmm] */
	char *help;		/* help/description string */
	struct item *next;	/* ptr to next item struct */
	struct item *prev;	/* ptr to previous item struct */	
};

/*---------------------------------------------------------------------------*
 *	forward declaration of individual initialized items
 *---------------------------------------------------------------------------*/

struct item i_ret, i_attr, i_pres, i_fkey, i_link, i_point, i_pager, i_editor,
		i_delim, i_wild, i_wldon, i_cdrom, i_cdnop, i_dotfn, i_tagmv,
		i_followsdir, i_autoexpand;

#ifdef USE_OWN_SYSTEM
struct item i_shell;
#endif

#define FIRST_I		(&i_ret)	/* first menu entry, upper left */

/*---------------------------------------------------------------------------*
 *	initialization of the doubly linked list for all menu items
 *---------------------------------------------------------------------------*/

#ifdef USE_OWN_SYSTEM
struct item i_ret = {
	2, 0, "   Return: ", 11, ITEM_ONOFF, (int *)&opt_return,
	"Specify wait for RETURN/ENTER key after executing an external command",
	&i_pres, &i_shell
};
#else
struct item i_ret = {
	2, 0, "   Return: ", 11, ITEM_ONOFF, (int *)&opt_return,
	"Specify wait for RETURN/ENTER key after executing an external command",
	&i_pres, &i_editor
};
#endif /* USE_OWN_SYSTEM */

struct item i_pres = {
	2, 26, " Preserve: ", 11, ITEM_ONOFF, (int *)&opt_preserve, 
	"Preserve current directory when moving up in directory tree",
	&i_link, &i_ret
};

struct item i_link = {
	2, 52, "SoftLinks: ", 11, ITEM_ONOFF, (int *)&opt_links,	
	"Enable detection and display of Softlinks in the attribute window",
	&i_point, &i_pres
};

struct item i_point = {
	4, 0, "      Dot: ", 11, ITEM_ONOFF, (int *)&opt_point, 
	"Enable to display the current directory as a dot",
	&i_tagmv, &i_link
};

struct item i_tagmv = {
	4, 26, "  TagMove: ", 11, ITEM_ONOFF, (int *)&opt_tagmove,	
	"Move to next filename after tagging the current one",
	&i_delim, &i_point
};

struct item i_delim = {
	4, 52, "Delimiter: ", 11, ITEM_CHAR, (int *)&opt_delimiter, 
	"Character to be used a a horizontal delimiter between windows",
	&i_cdrom, &i_tagmv
};

struct item i_cdrom = {
	6, 0, "   CD-Rom: ", 11, ITEM_ONOFF, (int *)&opt_cdrom,
	"Do special filename processing for cdrom filenames containing ';'",
	&i_cdnop, &i_delim
};

struct item i_cdnop = {
	6, 26, "CDNoProgs: ", 11, ITEM_ONOFF, (int *)&opt_cdnoprog, 		
	"Take all executable programs on CDROM as data files for display",
	&i_dotfn, &i_cdrom
};

struct item i_dotfn = {
	6, 52, " DotNames: ", 11, ITEM_ONOFF, (int *)&opt_dotnames,	
	"Display Filenames starting with a Dot",
	&i_wldon, &i_cdnop
};

struct item i_wldon = {
	8, 0, "WildEnabl: ", 11, ITEM_ONOFF, (int *)&opt_wildon, 	
	"Enable wildcard filename display filtering",
	&i_wild, &i_dotfn
};

struct item i_wild = {
	8, 26, " Wildcard: ", 11, ITEM_STRING, (int *)&opt_wild[0], 
	"Wildcard string for files being displayed in the file window",
	&i_followsdir, &i_wldon
};

struct item i_followsdir = {
	10, 0, "FollowSym: ", 11, ITEM_ONOFF, (int *)&opt_followsdir,	
	"Follow symbolic links to directories when stepping up (\"..\") in the filesystem",
	&i_autoexpand, &i_wild
};

struct item i_autoexpand = {
	10, 26, "AutoExpCm: ", 11, ITEM_ONOFF, (int *)&opt_autoexpand,	
	"Expand an aliased command onto the commandline immediately",
	&i_attr, &i_followsdir
};

struct item i_attr = {
	14, 0, "  Attribs: ", 11, ITEM_ONOFF | ITEM_NOCHANGE, (int *)&opt_attrib, 
	"Display the current file's attributes and Permissions (NOT CHANGEABLE)",
	&i_fkey, &i_followsdir
};

struct item i_fkey = {
	14, 40, "    Fkeys: ", 11, ITEM_ONOFF | ITEM_NOCHANGE, (int *)&opt_labels, 
	"Display virtual function key labels at bottom of the screen (NOT CHANGEABLE)",
	&i_pager, &i_attr
};

struct item i_pager = {
	16, 0, "    Pager: ", 11, ITEM_STRING | ITEM_NOCHANGE, (int *)&opt_more[0], 
	"External command for program to display text (NOT CHANGEABLE)",
	&i_editor, &i_fkey	
};

#ifndef USE_OWN_SYSTEM
struct item i_editor = {
	16, 40, "   Editor: ", 11, ITEM_STRING | ITEM_NOCHANGE, (int *)&opt_edit[0], 
	"External command for program to edit text (NOT CHANGEABLE)",
	&i_ret, &i_pager
};
#else
struct item i_editor = {
	16, 40, "   Editor: ", 11, ITEM_STRING | ITEM_NOCHANGE, (int *)&opt_edit[0], 
	"External command for program to edit text (NOT CHANGEABLE)",
	&i_shell, &i_pager
};

struct item i_shell = {
	18, 0, "    Shell: ", 11, ITEM_STRING | ITEM_NOCHANGE, (int *)&opt_shell[0], 	
	"Shell to be used to run all external commands/programs (NOT CHANGEABLE)",
	&i_ret, &i_editor
};
#endif /* USE_OWN_SYSTEM */

/*---------------------------------- EOF ------------------------------------*/
