/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	cwish.h - main header file
 *      --------------------------
 *
 *	Last Edit-Date: [Sat Jan  1 17:10:07 2000]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#ifdef M_XENIX
#include <sys/types.h>
#endif /* M_XENIX */

#ifdef USE_NCURSES
#include <ncurses.h>
/* #include <nterm.h> */	/* uncomment this for newer ncurses versions */
#include <term.h>
#else /* !USE_NCURSES */
#ifdef ultrix
#include <cursesX.h>
#else /* !ultrix */
#include <curses.h>
#endif /* ultrix */
#include <term.h>
#endif /* USE_NCURSES */

#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <grp.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <time.h>
#include <unistd.h>
#include <utmp.h>
#include <signal.h>

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <dirent.h>

#ifdef M_XENIX
#include <sys/sysmacros.h> 
#define index	strchr
#define rindex	strrchr
#define vfork	fork
#define MAXPATHLEN MAXNAMLEN

typedef int	gid_t;
typedef int	pid_t;
typedef int	uid_t;
typedef int	nlink_t;

/* Xenix has no symbolic links : */
#define lstat	stat 
#define readlink(a,b,c) (0)

#define S_ISFIFO(mode)  ((mode&0xF000) == 0x1000)
#define S_ISCHR(mode)   ((mode&0xF000) == 0x2000)
#define S_ISDIR(mode)   ((mode&0xF000) == 0x4000)
#define S_ISBLK(mode)   ((mode&0xF000) == 0x6000)
#define S_ISREG(mode)   ((mode&0xF000) == 0x8000)
#define S_ISLNK(mode)   ((mode&0xF000) == 0xa000)
#define S_ISSOCK(mode)  ((mode&0xF000) == 0xc000)
#endif /* M_XENIX */

#ifdef MAIN
#define EXTERNAL
#else
#define EXTERNAL extern
#endif

#define EXPSEL		'%'	/* expand selection character in commandline */

#define DEFDELIMCH 	'-'	/* default horizontal delimiter line char */

/* screen */

#define C_HEAD		0		/* copyright header line */
#define C_LINE		1		/* command entry line */
#define C_SEP		2		/* separator command & file display */
#define C_HEIGHT	3		/* commandline height in lines */

#define F_BEG		0		/* first line of file window */

#define A_SEP		0		/* separator filedisplay & status line */
#define A_LINE		1		/* attributes line */

#define CURDIRW		11		/* F_CURDIR: current dir string width */
#define ENTRYSW		9		/* F_CURDIR: entries string width */
#define ENTRYCW		4		/* F_CURDIR: entries count width */

/* positions in headerline */
#define CW_POS		33		/* cwish version	*/
#define MAIL_POS	52		/* mail message		*/
#define TIME_POS	62		/* date/time		*/

/* ALT_NORM - normal attribute line */

#define PP_PERM		0		/* position of permission string */

#define PP_DATE		11		/* position of date string */

#define PS_SIZE		26		/* position description string */
#define ST_SIZE		"Size:"		/* description */
#define PP_SIZE		PS_SIZE+6	/* position parameter string */

#define PS_LINK		42		/* position description string */
#define ST_LINK		"Link:"		/* description */
#define PP_LINK		PS_LINK+6	/* position parameter string */

#define PS_USER		52		/* position description string */
#define ST_USER		"Usr:"		/* description */
#define PP_USER		PS_USER+5	/* position parameter string */

#define PS_GRUP		66		/* position description string */
#define ST_GRUP		"Grp:"		/* description */
#define PP_GRUP		PS_GRUP+5	/* position parameter string */

/* ALT_TIME - 1st alternate attribute line */

#define APS_ADATE	0		/* position description string */
#define AST_ADATE	"Acc:"		/* description */
#define APP_ADATE	APS_ADATE+5	/* position parameter string */

#define APS_MDATE	20 		/* position description string */
#define AST_MDATE	"Mod:"		/* description */
#define APP_MDATE	APS_MDATE+5	/* position parameter string */

#define APS_CDATE	40		/* position description string */
#define AST_CDATE	"Chg:"		/* description */
#define APP_CDATE	APS_CDATE+5	/* position parameter string */

#define APS_INODE	60		/* position description string */
#define AST_INODE	"Ino:"		/* description */
#define APP_INODE	APS_INODE+5	/* position parameter string */

/* ALT_DEV - 2nd alternate attribute line */

#define APS_MAJ		0		/* position description string */
#define AST_MAJ		"Major:"	/* description */
#define APP_MAJ		APS_MAJ+7	/* position parameter string */

#define APS_MIN		13 		/* position description string */
#define AST_MIN		"Minor:"	/* description */
#define APP_MIN		APS_MIN+7	/* position parameter string */
#define APP_MINX	APS_MIN+18	/* position parameter string */

/* filename positioning and spacing */

#define STOPS 	 	30		/* # of positions for filename on screen */
#define MINSPACE 	2		/* minimum space between two filenames */
#define STARTCOL 	7		/* start column in a row */

#ifndef TRUE
#define TRUE		1		/* TRUE */
#define FALSE		0		/* not TRUE */
#endif

#define ABORT		-1		/* wildmatch */
#define GOOD		TRUE		/* good return */
#define BAD		FALSE		/* bad return */

#define FIRST		0x100		/* first entry of a file-page */
#define PAGE_MSK	0x0ff		/* for reading just the page */

/* some control characters */

#define CR		0x0d
#define LF		0x0a
#define TAB		0x09
#define BS		0x08
#define DEL		0x7f
#define SPACE		0x20
#define BEL		0x07
#define ESC		0x1b

/* keycodes for cursor motion */

#define	K_UP		0x10		/* ^P */
#define K_DOWN		0x0e		/* ^N */
#define	K_LEFT		0x02		/* ^B */
#define	K_RIGHT		0x06		/* ^F */
#define K_TAB		0x09		/* tab */
#define K_NEXT		0x16		/* ^V */
#define K_PREV		0x1a		/* ^Z */
#define K_QUIT		0x04		/* ^D */
#define K_REF		0x0c		/* ^L */
#define K_MARK		0x01		/* ^A */
#define	K_ECHO		0x17		/* ^W */
#define K_UMARK		0x15		/* ^U */

/* history mechanism */

#define HISLINES	32		/* no. of lines in history buffer */
#define HISLNLEN	512		/* length of one history line */

/* terminal types */

#define TERM_DUMB	0		/* dumb whatsoever */
#define TERM_HP		1		/* Hewlett Packard 2392 or 700/9x */
#define TERM_VT1	2		/* DEC VT100 */
#define TERM_VT2	3		/* DEC VT220 */
#define TERM_VT3	4		/* DEC VT320 */
#define TERM_PCVT	5		/* 386BSD pcvt driver */
#define TERM_HPX	6		/* hpterm X11 terminal emulator */

/* preserve options */

#define PRES_NO		0		/* don't preserve anything */
#define PRES_NORM	1		/* try "normal" position preserve */
#define PRES_DD		2		/* jump to cur dir after cd .. */

/* alternate attribute lines */

#define ALT_NORM	0		/* normal: perms, date, size, usr .. */
#define ALT_TIME	1		/*  times: access, mod, chg, inode   */
#define ALT_DEV		2		/* device: major, minor              */
#define ALT_MAX		2

/* one file */

struct onefile {		/* structure for one directory entry */
	off_t	osiz;		/* file size */
	uid_t	ousr;		/* user */
	gid_t	ogrp;		/* group */
	nlink_t	olinks;		/* no of hard links */	
	time_t	oatime;		/* access time */
	time_t	octime;		/* last file status chage time */
	ino_t	oinode;		/* inode */
	char	*onam;		/* filename, malloc'ed */
	char	oslink;		/* softlink flag */
	char	oprm[16];	/* permissions */
	char	odat[15];	/* modification time */
#ifdef USE_STRDEV
	dev_t	ominmaj;	/* minor and major number for device files  */
	char	ommvalid;	/* minor and major valid flag (-1 is used!) */
#endif /* USE_STRDEV */
	int	orow;		/* relative row to display item */
	int	ocol;		/* relative column to display item */
	int	tag;		/* flag for tagging files */
	int	page;		/* page and number for display window */
	struct onefile *next;	/* ptr to next entry */
	struct onefile *prev;	/* prt to previous entry */
};

/* onam[0] contains the entry type, which can be:	*/

#define ISDIR	'1'
#define	ISPROG	'2'
#define	ISDATA	'3'
#define	ISCHAR	'4'
#define	ISBLOCK	'5'
#define	ISPIPE	'6'
#define ISSOCK	'7'
#define ISUNKN	'8'

#define OPTSLEN	60		/* option string length */

#define WILDCHARS "\\?*["	/* special meaning chars */


#define YEAR(year) (year) > 99 ? ((year)-100) : (year)

/* fkey labels */

struct fk_tab {
	char *label_16;		/* HP Terminal Labels */
	char *label_8;		/* DEC (and others) Terminal Labels */
};

extern struct fk_tab sys_keys[];

#ifdef MAIN

char *hislines[HISLINES];	/* array of history line pointers */
int  cur_his = 0;		/* current history line */
char cbuff[HISLNLEN+16];	/* command buffer */

char counter[5] = "\0";		/* entrycount as string */
char cur_path[MAXPATHLEN+256];	/* current path - REAL, without symlink */
char lcur_path[MAXPATHLEN+256] = "";	/* current path - following symlinks */
char errorline[256];		/* error line if errorflag == 1 */
char headerline[256];		/* headerline for copyright etc */

int stops[STOPS];		/* table of stop points on row */

int tagsize = 0;		/* sum of sizes of tagged files */

struct onefile *cur_file = NULL;/* the CURRENT highlighted filename */
struct onefile *first = NULL;	/* init dir-list head-ptr */
struct onefile *last = NULL;	/* init dir-list tail-ptr */

int maxfnleng = 0;		/* length of longest filename of this dir */
int opt_attrib = 1;		/* display attribute line */
int opt_labels = 0;		/* use virtual f-key labels in last line */
int opt_point  = 0;		/* display current dir in file window */
int opt_preserve = 0;		/* initially preserve dirs when cd .. */
int opt_links = 0;		/* initially show files not links */
int opt_return = 1;		/* press any key to continue */
unsigned char opt_delimiter = 0;/* window delimiter character */
int opt_wildon = 0;		/* no wildcarding yet */
char opt_wild[OPTSLEN+1];	/* file window wildcard spec */
char opt_edit[OPTSLEN+1];	/* user sepecified editor */
char opt_more[OPTSLEN+1];	/* user sepecified pager */

#ifdef USE_OWN_SYSTEM
char opt_shell[OPTSLEN+1];	/* user sepecified shell */
#endif

int opt_cdrom = 0;		/* cdrom special filename processing */
int opt_cdnoprog = 0;		/* cdrom, convert executables to data */
int opt_dotnames = 1;		/* display filenames starting with dot */
int opt_tagmove = 1;		/* move to next filename after tagging */
int opt_altattr = ALT_NORM;	/* alternate attribute line */

#ifndef M_XENIX
int opt_followsdir = 1;		/* follow symbolic linked directories */
#else /* M_XENIX */
int opt_followsdir = 0;		/* SCO Xenix has no symbolic links */
#endif /* M_XENIX */

int opt_autoexpand = 0;		/* auto expand aliases on commandline */

char *etab[50][2];		/* endings table for tail config keyword */
char *atab[50][2];		/* aliases table for alias config keyword */

#else /* !MAIN */

extern char *hislines[];
extern int  cur_his;

extern char cbuff[];

extern char counter[];
extern char cur_path[];
extern char lcur_path[];
extern char errorline[];
extern char headerline[];

extern int stops[];

extern int tagsize;

extern struct onefile *cur_file;
extern struct onefile *first;
extern struct onefile *last;

extern int maxfnleng;
extern int opt_attrib;
extern int opt_labels;
extern int opt_point;
extern int opt_preserve;
extern int opt_links;
extern int opt_return;
extern unsigned char opt_delimiter;
extern int opt_wildon;
extern char opt_wild[];
extern char opt_edit[];
extern char opt_more[];

#ifdef USE_OWN_SYSTEM
extern char opt_shell[];
#endif

extern int opt_cdrom;
extern int opt_cdnoprog;
extern int opt_dotnames;
extern int opt_tagmove;
extern int opt_altattr;
extern int opt_followsdir;
extern int opt_autoexpand;

extern char *etab[50][2];
extern char *atab[50][2];

#endif /* MAIN */

EXTERNAL char bschar;		/* backspace */

EXTERNAL WINDOW *attr_w;	/* attributes display window (separation,attributes) */
EXTERNAL WINDOW *cmnd_w;	/* command line window (copyright,command,separation) */
EXTERNAL WINDOW *file_w;	/* files display window (file names) */
EXTERNAL WINDOW *fst_w;		/* files status window (cur dir, entries) */
EXTERNAL WINDOW *flbl_w;	/* function key labels */

EXTERNAL char *envhome;		/* ptr to HOME - env var */
EXTERNAL char *envmail;		/* ptr to HOME - env var */
EXTERNAL char *envmore;		/* ptr to PAGER - env var */
EXTERNAL char *envedit;		/* ptr to EDITOR - env var */
EXTERNAL char *term_string;	/* terminal type string from environment */

#ifdef USE_OWN_SYSTEM
EXTERNAL char *envshell;	/* ptr to SHELL - env var */
#endif

EXTERNAL int termtype;		/* flag what terminal category we are on */

EXTERNAL struct onefile *scr_beg;/* ptr to first filename on screen */
EXTERNAL struct onefile *scr_end;/* ptr to last filename on screen */

EXTERNAL int cur_page;		/* current file window page */
EXTERNAL int errorflag;		/* if error string is in header line */
EXTERNAL int fileheight;	/* height of filewindow */
EXTERNAL struct fk_tab (*cur_fktab)[]; /* current fkey label table */
EXTERNAL int iscdfs;		/* flag, true if we are on a cdrom filesys */

EXTERNAL char overstrike;	/* commandline overstrike mode */

void alias_cmd( char *buffer );
void attribs ( int flag );
void bol_line ( void );
int cd( char *string );
void clear_toeol ( void );
void close_cdir ( void );
void clrerror ( void );
int cmdline ( int c );
void complete ( void );
void config( void );
int cr_on_files ( void );
int curcol ( void );
void cur_blink ( struct onefile *current );
void cur_inv ( struct onefile *current );
void cur_norm ( struct onefile *current );
void del_char ( void );
void dis_hist ( void );
void edit_current ( void );
void eol_line ( void );
void error ( char *str );
void exec_command ( char *cline );
void expand_alias(void);
void expand_tilde(void);
void expsel ( char *p );
void fatal ( char *str );
int fill_list ( void );
void fini_flabels ( void );
int firstfile ( void );
void fnclabel ( int n, char *string );
void free_list ( void );
void fresh_files ( void );
#ifndef HAS_USR_GRP
char *group_from_gid ( gid_t gid, int size );
#endif
void handlebs ( void );
void handlecr ( void );
void header ( void );
void help ( void );
void h_files ( void );
void h_line ( void );
void init_files ( int preserve, char *dirname );
void init_flabels ( void );
void init_header ( void );
void init_history ( void );
void init_screen ( void );
void init_time ( void );
void is_tagged ( struct onefile *current );
int lastfile ( void );
void left_line ( void );
int main ( int argc, char *argv[] );
void move_down ( void );
void move_hmdn ( void );
void move_home ( void );
void move_left ( void );
void move_right ( void );
void move_up ( void );

#ifdef USE_OWN_SYSTEM
int mysystem(const char *cmdstring);
#endif

void name_echo ( void );
int nextfile ( void );
void next_line ( void );
void next_page ( void );
int prevfile ( void );
void prev_line ( void );
void prev_page ( void );
void readrc ( void );
void resume_time ( void );
void right_line ( void );
void save_line ( void );
void sepaline ( struct _win_st *window );
void set_termtype ( void );
struct onefile *store ( struct onefile *new, struct onefile *top );
void suspend_time ( void );
char *system_name( void );
void tail_cmd( char *buffer );
void tag_current ( struct onefile *current );
void timeout_hdlr ( int sig );
void untag_all ( void );
void update_all ( void );
void update_files ( void );
void usage ( void );

#ifndef HAS_USR_GRP
char *user_from_uid ( uid_t uid, int size );
#endif

int wildmat( char *text, char *p );
void yank ( void );

void left_linew(void);
void right_linew(void);

/*---------------------------------- EOF -------------------------------------*/
