/*
 *
 * Copyright (c) 1988, 2000 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	files.c - file handling routines
 *      --------------------------------
 *
 *	Last Edit-Date: [Sat Jan  1 17:14:49 2000]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"				/* global include file */

#ifdef ultrix

#include <sys/fs_types.h>

#define S_IFLNK		0120000			/* symbolic link */
#define S_IFSOCK	0140000			/* socket */
#define _S_IFLNK	S_IFLNK
#define _S_IFSOCK	S_IFSOCK

/* macro to test for symbolic link */
#define S_ISLNK( mode )		(((mode) & _S_IFLNK) == _S_IFLNK)

/* macro to test for socket */
#define S_ISSOCK( mode )	(((mode) & _S_IFSOCK) == _S_IFSOCK)

#else /* !ultrix */

#if !defined(__386BSD__) && !defined(__NetBSD__) && !defined(__FreeBSD__)

#ifndef NOSTATFS
#include <sys/vfs.h>
#endif /* NOSTATFS */

/* some SYSVR4 don't know these */

#ifndef S_ISLNK
#define S_ISLNK(_Mode)		(((_Mode) & S_IFMT) == S_IFLNK)
#endif

#ifndef S_ISSOCK
#define S_ISSOCK(_Mode)		(((_Mode) & S_IFMT) == S_IFSOCK)
#endif

#endif /* !defined(__386BSD__) && !defined(__NetBSD__) && !defined(__FreeBSD__) */

#endif /* ultrix */

#include <sys/mount.h>

/* is there a better way to do this ??? */

#ifndef S_IRUSR
#define S_IRUSR 0400				/* owner read permission */
#endif
#ifndef S_IWUSR
#define S_IWUSR 0200				/* owner write permission */
#endif
#ifndef S_IXUSR
#define S_IXUSR 0100				/* owner execute permission */
#endif

#ifndef S_IRGRP
#define S_IRGRP 0040				/* group read permission */
#endif
#ifndef S_IWGRP
#define S_IWGRP 0020				/* group write permission */
#endif
#ifndef S_IXGRP
#define S_IXGRP 0010				/* group execute permission */
#endif

#ifndef S_IROTH
#define S_IROTH 0004				/* others read permission */
#endif
#ifndef S_IWOTH
#define S_IWOTH 0002				/* others write permission */
#endif
#ifndef S_IXOTH
#define S_IXOTH 0001				/* others execute permission */
#endif

static int pia[] =
{
	S_IFDIR,
	S_IRUSR, S_IWUSR, S_IXUSR,
	S_IRGRP, S_IWGRP, S_IXGRP,
	S_IROTH, S_IWOTH, S_IXOTH
};

static char pca[] = "drwxrwxrwx";

/*---------------------------------------------------------------------------*
 *	create a doubly linked list in sorted order, return pointer to new
 *	first element of list
 *---------------------------------------------------------------------------*/
struct onefile *store
  (register struct onefile *new,		/* new entry to store into list */
   register struct onefile *top)		/* current first entry in list */
{
	register struct onefile *old, *p;

	if (last == NULL)			/* enter very first element ? */
	{
		new->next = NULL;
		new->prev = NULL;
		last = new;			/* init last */
		return (new);			/* return new first */
	}
	p = top;				/* p = old first element */
	old = NULL;
	while (p)
	{
		if ((strcmp(p->onam, new->onam)) < 0)	/* current less new ? */
		{
			old = p;
			p = p->next;
		}
		else
		{				/* current >= new */

			if (p->prev)
			{
				p->prev->next = new;
				new->next = p;
				new->prev = p->prev;
				p->prev = new;
				return (top);
			}
			new->next = p;
			new->prev = NULL;
			p->prev = new;
			return (new);
		}
	}
	old->next = new;
	new->next = NULL;
	new->prev = old;
	last = new;
	return (first);
}

/*---------------------------------------------------------------------------*
 *	read current directory and build up a doubly linked sorted list
 *---------------------------------------------------------------------------*/
int
fill_list(void)
{
	register struct dirent *dp;
	register struct onefile *new_entry;
	register struct tm *tp;
	register DIR *dirp;

	static struct stat fdbuffer;
	static char templine[MAXPATHLEN + 2];

	int templeng;
	int flcnt = 0;
	int sret;

	maxfnleng = 0;				/* init global variables */

	iscdfs = 0;				/* no cdrom filesys */

#ifndef NOSTATFS

	if (opt_cdrom)
	{

#if defined(__386BSD__) || defined (__NetBSD__) || defined(__FreeBSD__)

/* this happened after Net-2 -> BSD4.4 upgrade */
#ifndef MOUNT_ISOFS
#define MOUNT_ISOFS MOUNT_CD9660
#endif

		struct statfs statfsb;

		/* the 386BSD way, this also seems to work for SunOS 4.1.x */
		if ((statfs(".", &statfsb) == 0) && (statfsb.f_type == MOUNT_ISOFS))

#else /* ! __386BSD__  etc.. */

#ifdef ultrix

		struct fs_data statfsb;

		/* the ULTRIX way ... */
		if ((statfs(".", &statfsb) == 0) && (statfsb.fd_req.fstype == GT_CDFS))

#else /* ! ultrix */

		struct statfs statfsb;

		/* the HPUX way ... */
		if ((statfs(".", &statfsb) == 0) && (statfsb.f_fsid[1] == MOUNT_CDFS))

#endif /* ultrix */

#endif /* defined(__386BSD__) || defined (__NetBSD__) || defined(__FreeBSD__) */

			iscdfs = 1;
	}

#endif /* NOSTATFS */

	if ((dirp = opendir(".")) == NULL)
		return (-1);			/* cannot open directory */

	for (dp = readdir(dirp); dp != NULL; dp = readdir(dirp))
	{
		flcnt++;			/* increment file count */

		if (!opt_point)			/* display dot ? */
		{
			if ((*(dp->d_name) == '.') && (*(dp->d_name + 1) == '\0'))
				continue;
		}

		if (!opt_dotnames)		/* display dot-names ? */
		{
			if ((*(dp->d_name) == '.') && (*(dp->d_name + 1) != '.') && (*(dp->d_name + 1) != '\0'))
				continue;
		}

		if ((new_entry = (struct onefile *) malloc(sizeof(struct onefile))) == NULL)
		{
			fatal("files.c, fill_list(): structure onefile malloc failed");
		}

		new_entry->oslink = 0;		/* init softlink flag */

		if (opt_links)			/* detect SOFT links ? */
		{
			sret = 0;		/* init return */

			if ((lstat(dp->d_name, &fdbuffer)) < 0)
			{
				free(new_entry);	/* lstat failed */
				continue;	/* next one .. */
			}

			if (S_ISLNK(fdbuffer.st_mode))	/* have a soft link ? */
			{
				struct stat tstfdbuffer;

				new_entry->oslink = 1;	/* mark as soft link */

				/* detect & display links where the */
				/*   object they point to is absent */

				if ((sret = stat(dp->d_name, &tstfdbuffer)) == 0)
					fdbuffer = tstfdbuffer;		/* dest present */
				else
					sret = 0;	/* fake entry */
			}
		}
		else
		{
			sret = stat(dp->d_name, &fdbuffer);	/* get status */
		}

		if (sret < 0)			/* return from stat bad ? */
		{
			free(new_entry);	/* next entry ... */
			continue;
		}
		else
			/* got status, fill in from stat buffer */
		{
			register int j;

			/* size, userid, groupid, hardlinks, inode */

			new_entry->osiz = fdbuffer.st_size;
			new_entry->ousr = fdbuffer.st_uid;
			new_entry->ogrp = fdbuffer.st_gid;
			new_entry->olinks = fdbuffer.st_nlink;
			new_entry->oinode = fdbuffer.st_ino;

#ifdef USE_STRDEV
			new_entry->ommvalid = 0;
#endif /* USE_STRDEV */

			/* permissions */

			for (j = 0; j < 10; j++)
			{
				if (fdbuffer.st_mode & pia[j])
					new_entry->oprm[j] = pca[j];
				else
					new_entry->oprm[j] = '-';
			}
			new_entry->oprm[j] = '\0';	/* delimit perms */

			/* type of file -> templine[0] */

			if (S_ISREG(fdbuffer.st_mode))
			{
				/* if wildmatch, check for match */

				if (opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				new_entry->oprm[0] = '-';	/* regular file */

				if ((new_entry->oprm[3] == 'x') ||
				    (new_entry->oprm[6] == 'x') ||
				    (new_entry->oprm[9] == 'x'))
				{
					if (iscdfs && opt_cdnoprog)
					{
						new_entry->oprm[3] = '-';
						new_entry->oprm[6] = '-';
						new_entry->oprm[9] = '-';
						templine[0] = ISDATA;
					}
					else
					{
						templine[0] = ISPROG;
					}
				}
				else
					templine[0] = ISDATA;
			}
			else if (S_ISDIR(fdbuffer.st_mode))
			{
				if (opt_wildon)
				{
					flcnt--;
				}
				templine[0] = ISDIR;
				new_entry->oprm[0] = 'd';
			}
			else if (S_ISCHR(fdbuffer.st_mode))
			{
				if (opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISCHAR;
				new_entry->oprm[0] = 'c';
#ifdef USE_STRDEV
				new_entry->ominmaj = fdbuffer.st_rdev;
				new_entry->ommvalid = 1;
#endif /* USE_STRDEV */
			}
			else if (S_ISBLK(fdbuffer.st_mode))
			{
				if (opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISBLOCK;
				new_entry->oprm[0] = 'b';
#ifdef USE_STRDEV
				new_entry->ominmaj = fdbuffer.st_rdev;
				new_entry->ommvalid = 1;
#endif /* USE_STRDEV */
			}
			else if (S_ISFIFO(fdbuffer.st_mode))
			{
				if (opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISPIPE;
				new_entry->oprm[0] = 'p';
			}
			else if (S_ISSOCK(fdbuffer.st_mode))
			{
				if (opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISSOCK;
				new_entry->oprm[0] = 'n';
			}
			else
			{
				if (opt_wildon && (wildmat(dp->d_name, opt_wild) != TRUE))
				{
					free(new_entry);
					continue;
				}
				templine[0] = ISUNKN;	/* unknown */
				new_entry->oprm[0] = 'u';
			}
			templine[1] = '\0';	/* terminate templine */

			/* check for setuid */

			if (fdbuffer.st_mode & S_ISUID)
			{
				if (new_entry->oprm[3] == 'x')
					new_entry->oprm[3] = 's';
				else
					new_entry->oprm[3] = 'S';
			}

			/* check for setgid */

			if (fdbuffer.st_mode & S_ISGID)
			{
				if (new_entry->oprm[6] == 'x')
					new_entry->oprm[6] = 's';
				else
					new_entry->oprm[6] = 'S';
			}

			/* check for sticky */

#if defined(S_ISVTX) || defined(S_ISTXT)

#if defined(S_ISVTX)
			if (fdbuffer.st_mode & S_ISVTX)
#elif defined(S_ISTXT)
			if (fdbuffer.st_mode & S_ISTXT)
#endif /* defined(S_ISVTX) */

				{
					if (new_entry->oprm[9] == 'x')
						new_entry->oprm[9] = 't';
					else
						new_entry->oprm[9] = 'T';
				}
#endif /* defined(S_ISVTX) || defined(S_ISTXT) */

			/* file time */

			tp = localtime(&(fdbuffer.st_mtime));

#ifdef DATE_US
			/* us date string */
			sprintf(new_entry->odat, "%02d/%02d/%02d %02d:%02d",
				(tp->tm_mon) + 1, tp->tm_mday,
				YEAR(tp->tm_year),
				tp->tm_hour, tp->tm_min);
#else
			/* german date string */
			sprintf(new_entry->odat, "%02d.%02d.%02d %02d:%02d",
				tp->tm_mday, (tp->tm_mon) + 1,
				YEAR(tp->tm_year),
				tp->tm_hour, tp->tm_min);
#endif /* DATE_US  */

			new_entry->oatime = fdbuffer.st_atime;
			new_entry->octime = fdbuffer.st_ctime;

			/* misc init */

			new_entry->tag = 0;	/* clear tag-field */
			new_entry->page = -1;	/* page # unknown */

			/* new max filename length */

			templeng = strlen(dp->d_name);	/* get name length */
			if (templeng > maxfnleng)	/* new max filename length ? */
				maxfnleng = templeng;	/* yes */

			/* cat templine + filename */

			strcat(templine, dp->d_name);	/* file name */

			/* alloc filename memory and copy name into it */

			if ((new_entry->onam = (char *) malloc(strlen(templine) + 1)) == NULL)
			{
				fatal("files.c, fill_list(): malloc filename string memory failed");
			}

			strcpy(new_entry->onam, templine);

			/* sort entry into linked list */

			first = store(new_entry, first);
		}
	}
	closedir(dirp);				/* close current dir */
	sprintf(counter, "%4d", flcnt);		/* entries counter */
	return (0);				/* ok return */
}

/*---------------------------------------------------------------------------*
 *	free the current malloc'ed list
 *---------------------------------------------------------------------------*/
void
free_list(void)
{
	register struct onefile *dir;
	register struct onefile *tmp;

	tagsize = 0;				/* reset any tags/marks size */

	dir = first;				/* start of linked list */

	while (dir)				/* free all */
	{
		tmp = dir->next;		/* save ptr to next entry */
		free(dir->onam);		/* free filename space */
		free(dir);			/* free struct space */
		dir = tmp;			/* ptr = ptr to next entry */
	}
	first = NULL;				/* first ptr = NULL */
	last = NULL;				/* last ptr = NULL */
}

/*---------------------------------- EOF -------------------------------------*/
