/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	fkeys.c - terminal function key handling
 *      ----------------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:03:53 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

/*---------------------------------------------------------------------------*
 *	write string to function key label
 *---------------------------------------------------------------------------*/
void
fnclabel(int n, char *string)
{
	char xmit[5];				/* transmitted string ( ESC 'p'...'w' ) */
	char result[50];			/* result buffer */

	/* check if we are on an hp-terminal or lookalike */

	if (termtype == TERM_HP || termtype == TERM_PCVT || termtype == TERM_HPX)
	{
		if (n < 1 || n > 8)		/* and it must be a legal key number */
			return;
		if ((strlen(string)) != 16)
			return;

		xmit[0] = ESC;
		xmit[1] = 'o' + n;
		xmit[2] = '\0';
		sprintf(result, "%c&f0a%dk16d2L%s%s", ESC, n, string, xmit);
		write(2, result, strlen(result));
		sprintf(result, "%c&jB", ESC);	/* display user function keys */
		write(2, result, strlen(result));
		return;
	}
	else if (opt_labels)			/* if virtual f-key labels .. */
	{
		if (n < 1 || n > 8)		/* and it must be a legal key number */
			return;

		if ((strlen(string)) != 9)
			return;

		wmove(flbl_w, 0, ((n - 1) * 10));	/* last line */

		wattron(flbl_w, A_REVERSE);
		waddstr(flbl_w, string);
		wattroff(flbl_w, A_REVERSE);
		wrefresh(flbl_w);
	}
}

/*---------------------------------------------------------------------------*
 *	function key labels initialization
 *---------------------------------------------------------------------------*/
void
init_flabels(void)
{
	int i;

	for (i = 0; i < 8; i++)
	{
		if (termtype == TERM_HP || termtype == TERM_PCVT || termtype == TERM_HPX)
			fnclabel(i + 1, (*cur_fktab)[i].label_16);
		else if (opt_labels)
			fnclabel(i + 1, (*cur_fktab)[i].label_8);
		else
			break;
	}
}

/*---------------------------------------------------------------------------*
 *	restore function key labels
 *---------------------------------------------------------------------------*/
void
fini_flabels(void)
{
	static char modes[] =
	{ESC, '&', 'j', 'A', '\0'};

	/* check if we are on an hp-terminal */

	if (termtype == TERM_HP || termtype == TERM_PCVT || termtype == TERM_HPX)
		write(2, modes, strlen(modes));
}

/*---------------------------------------------------------------------------*
 *	system function keys table
 *---------------------------------------------------------------------------*/

struct fk_tab sys_keys[] =
{
	{"  Edit    File  ", "Edit File"},
	{"  Echo  Filename", "Echo Name"},
	{"  Help  Commands", "Help Cmds"},
	{" Config  Screen ", "Conf Scrn"},
	{"  Mark  Filename", "Mark File"},
	{" Unmark allmarkd", "UnmarkAll"},
	{"AlternatAttrLine", "Alt AttLn"},
	{" Close   CurDir ", "Close Dir"}
};

/*---------------------------------- EOF -------------------------------------*/
