/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	header.c - window header and separator line handling
 *      ----------------------------------------------------
 *
 *	Last Edit-Date: [Sat Jan  1 17:16:06 2000]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

#ifdef HAS_UNAME
#include <sys/utsname.h>
#endif

#if defined (USE_STRDEV) && defined (__hpux)
#include <sys/sysmacros.h>
#endif

#ifdef USE_STRDEV
static char *bin_str(unsigned long val, int length);
#endif

/*---------------------------------------------------------------------------*
 *	print a horizontal separation line based on delimiter-character
 *---------------------------------------------------------------------------*/
void
sepaline(WINDOW * window)
{
	int i = COLS;
	unsigned char delim;

	if (window == cmnd_w)
		wmove(window, 2, 0);
	else if (window == attr_w)
		wmove(window, 0, 0);
	else
		return;

	if (opt_delimiter)
	{
		delim = opt_delimiter;
	}
	else if (enter_alt_charset_mode && exit_alt_charset_mode &&
		 *enter_alt_charset_mode && *exit_alt_charset_mode)
	{
		wattron(window, A_ALTCHARSET);
		switch (termtype)
		{
			case TERM_HP:		/* HEWLETT-PACKARD Terminals (2392,700/9x etc) */
				delim = ';';
				break;

			case TERM_VT2:		/* DEC VT220 / 320 */
			case TERM_VT3:
			case TERM_PCVT:
				delim = 'q';
				break;

			case TERM_HPX:		/* X11 HEWLETT-PACKARD */
				delim = '-';
				break;

			default:		/* everything else */
				delim = DEFDELIMCH;
				break;
		}
	}
	else
	{
		delim = DEFDELIMCH;
	}

	while (i--)
		waddch(window, delim);

	if (enter_alt_charset_mode && exit_alt_charset_mode && !opt_delimiter &&
	    *enter_alt_charset_mode && *exit_alt_charset_mode)
	{
		wattroff(window, A_ALTCHARSET);
	}
}

/*---------------------------------------------------------------------------*
 *	print error string in header-line
 *---------------------------------------------------------------------------*/
void
error(char *str)
{
	extern int errno;
#ifndef OWN_SYS_ERRLIST
	extern char *sys_errlist[];
#endif
	extern int sys_nerr;

	int i;

	if (str == NULL)
	{
		if (errno && (errno <= sys_nerr))
			strcpy(errorline, sys_errlist[errno]);
		else
			strcpy(errorline, "Unknown Error, call your guru ...");
	}
	strcpy(errorline, str);

	i = strlen(errorline);
	while (i++ < COLS)
		strcat(errorline, " ");

	wmove(cmnd_w, C_HEAD, 0);
	wattron(cmnd_w, A_REVERSE);
	waddstr(cmnd_w, errorline);
	wattroff(cmnd_w, A_REVERSE);
	wmove(cmnd_w, C_LINE, curcol());
	suspend_time();
	errorflag = 1;
}

/*---------------------------------------------------------------------------*
 *	clear error string and rewrite "normal" header
 *---------------------------------------------------------------------------*/
void
clrerror(void)
{
	if (errorflag)				/* error-string in header ? */
	{
		resume_time();
		errorflag = 0;			/* reset error-flag */
		wmove(cmnd_w, C_HEAD, 0);	/* headerline pos */
		wclrtoeol(cmnd_w);		/* clear old */
		header();			/* rewrite header */
		wmove(cmnd_w, C_LINE, curcol());
	}
}

/*---------------------------------------------------------------------------*
 *	fatal, irrecoverable error
 *---------------------------------------------------------------------------*/
void
fatal(char *str)
{
	free_list();				/* free memory */
	fini_flabels();				/* fk-labels to normal */
	endwin();				/* exit curses */
	fprintf(stderr, "\n\n\t *** cwish Fatal Error: ***\n\t *** [%s] ***\n\n", str);
	exit(1);
}

/*---------------------------------------------------------------------------*
 *	print headerline centered in display
 *---------------------------------------------------------------------------*/
void
header(void)
{
	int p, q;

	wmove(cmnd_w, C_HEAD, 0);

	if (errorflag)
	{
		wattron(cmnd_w, A_REVERSE);
		waddstr(cmnd_w, errorline);
		wattroff(cmnd_w, A_REVERSE);
	}
	else
	{
		wattron(cmnd_w, A_REVERSE);
		if ((COLS + 1) > 80)
		{
			p = (COLS + 1) - 80;
			p /= 2;
			q = p;
			while (q--)
				waddch(cmnd_w, SPACE);
			waddstr(cmnd_w, headerline);
			q = p;
			while (q--)
				waddch(cmnd_w, SPACE);
		}
		else
		{
			waddstr(cmnd_w, headerline);
		}
		wattroff(cmnd_w, A_REVERSE);
	}
	sepaline(cmnd_w);
	wmove(cmnd_w, C_LINE, curcol());
}

/*---------------------------------------------------------------------------*
 *	display the attribute window, init if required
 *---------------------------------------------------------------------------*/
void
attribs(int flag)
{
	if (!opt_attrib)
		return;

	if (flag)				/* rewrite static data ?? */
	{
		sepaline(attr_w);		/* draw horiz line */
		wmove(attr_w, 1, 0);		/* start of line */
		wclrtoeol(attr_w);		/* clear old */

		if (opt_altattr == ALT_NORM)	/* normal */
		{
			wmove(attr_w, 1, PS_SIZE);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, ST_SIZE);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, PS_USER);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, ST_USER);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, PS_GRUP);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, ST_GRUP);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, PS_LINK);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, ST_LINK);
			wattroff(attr_w, A_REVERSE);
		}
		else if (opt_altattr == ALT_TIME)	/* times */
		{
			wmove(attr_w, 1, APS_ADATE);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, AST_ADATE);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, APS_MDATE);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, AST_MDATE);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, APS_CDATE);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, AST_CDATE);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, APS_INODE);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, AST_INODE);
			wattroff(attr_w, A_REVERSE);
		}

#ifdef USE_STRDEV
		else
		{
			wmove(attr_w, 1, APS_MAJ);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, AST_MAJ);
			wattroff(attr_w, A_REVERSE);

			wmove(attr_w, 1, APS_MIN);
			wattron(attr_w, A_REVERSE);
			waddstr(attr_w, AST_MIN);
			wattroff(attr_w, A_REVERSE);
		}
#endif /* USE_STRDEV */

	}
	if (opt_altattr == ALT_NORM)		/* normal */
	{
		mvwaddstr(attr_w, 1, PP_PERM, cur_file->oprm);	/* permissions */
		if (cur_file->oslink)		/* a soft link ?? */
			mvwaddch(attr_w, 1, PP_PERM, 'l');	/*     yes ! */
		mvwaddstr(attr_w, 1, PP_DATE, cur_file->odat);	/* date */
		mvwprintw(attr_w, 1, PP_SIZE, "%-9d", cur_file->osiz);	/* filesize */
#ifdef NOTDEF
		mvwaddstr(attr_w, 1, PP_USER, user_from_uid(cur_file->ousr, 8));	/* user */
		mvwaddstr(attr_w, 1, PP_GRUP, group_from_gid(cur_file->ogrp, 8));	/* group */
#else
		mvwprintw(attr_w, 1, PP_USER, "%-8s", user_from_uid(cur_file->ousr, 8));	/* user */
		mvwprintw(attr_w, 1, PP_GRUP, "%-8s", group_from_gid(cur_file->ogrp, 8));	/* group */
#endif
		mvwprintw(attr_w, 1, PP_LINK, "%-2d", cur_file->olinks);	/* hard links */
	}
	else if (opt_altattr == ALT_TIME)	/* times */
	{
		register struct tm *tp;
		static char buffer[16];

		tp = localtime(&(cur_file->oatime));

#ifdef DATE_US
		sprintf(buffer, "%02d/%02d/%02d %02d:%02d",
			(tp->tm_mon) + 1, tp->tm_mday,
			YEAR(tp->tm_year),
			tp->tm_hour, tp->tm_min);
#else
		sprintf(buffer, "%02d.%02d.%02d %02d:%02d",
			tp->tm_mday, (tp->tm_mon) + 1,
			YEAR(tp->tm_year),
			tp->tm_hour, tp->tm_min);
#endif /* DATE_US  */

		mvwaddstr(attr_w, 1, APP_ADATE, buffer);	/* access date */

		mvwaddstr(attr_w, 1, APP_MDATE, cur_file->odat);	/* modify date */

		tp = localtime(&(cur_file->octime));

#ifdef DATE_US
		sprintf(buffer, "%02d/%02d/%02d %02d:%02d",
			(tp->tm_mon) + 1, tp->tm_mday,
			YEAR(tp->tm_year),
			tp->tm_hour, tp->tm_min);
#else
		sprintf(buffer, "%02d.%02d.%02d %02d:%02d",
			tp->tm_mday, (tp->tm_mon) + 1,
			YEAR(tp->tm_year),
			tp->tm_hour, tp->tm_min);
#endif /* DATE_US */

		mvwaddstr(attr_w, 1, APP_CDATE, buffer);	/* stat change date */
		mvwprintw(attr_w, 1, APP_INODE, "%-10d", cur_file->oinode);	/* inode */
	}
#ifdef USE_STRDEV
	else
	{
		if (cur_file->ommvalid == 0)
		{
			mvwprintw(attr_w, 1, APP_MAJ, "void ");
			mvwprintw(attr_w, 1, APP_MIN, "void ");
		}
		else
		{
			mvwprintw(attr_w, 1, APP_MAJ, "%-5d",
				  major(cur_file->ominmaj));
			mvwprintw(attr_w, 1, APP_MIN, "%8d ",
				  minor(cur_file->ominmaj));
#ifdef __hpux	/* dev_t is long, upper 8bit major, lower 24bit minor */
			mvwprintw(attr_w, 1, APP_MINX, "0x%6.6x   %s",
				  minor(cur_file->ominmaj),
				  bin_str(cur_file->ominmaj, 24));
#else		/* dev_t is short, upper 8bit major, lower 8bit minor */
			mvwprintw(attr_w, 1, APP_MINX, "0x%2.2x   %s",
				  minor(cur_file->ominmaj),
				  bin_str(cur_file->ominmaj, 8));
#endif
		}
		wclrtoeol(attr_w);
	}
#endif /* USE_STRDEV */
}

/*---------------------------------------------------------------------------*
 *	return ptr to system name
 *---------------------------------------------------------------------------*/
char *
system_name(void)
{
	static char buffer[80];
	char *p;

#if defined(HAS_GETHOSTNAME)

	if (gethostname(buffer, 79) == -1)
		strcpy(buffer, "unknown");

#elif defined(HAS_UNAME)

	struct utsname utsname;

	if (uname(&utsname) == -1)
		strcpy(buffer, "unknown");
	else
		strcpy(buffer, utsname.nodename);

#elif defined(MY_HOSTNAME)

	strcpy(buffer, MY_HOSTNAME);

#else

	strcpy(buffer, "unknown");

#endif

#ifdef NO_FQDN
	if ((p = index(buffer, '.')) != NULL)
		*p = '\0';
#endif

	return (buffer);
}

#ifdef USE_STRDEV
/*---------------------------------------------------------------------------*
 *	return ptr to string of 1's and 0's for value
 *---------------------------------------------------------------------------*/
static char *
bin_str(unsigned long val, int length)
{
	static char buffer[80];
	int i = 0;

	if (length > 32)
		length = 32;

	val = val << (32 - length);

	while (length--)
	{
		if (val & 0x80000000)
			buffer[i++] = '1';
		else
			buffer[i++] = '0';
		if ((length % 4) == 0 && length)
			buffer[i++] = '.';
		val = val << 1;
	}
	return (buffer);
}
#endif

/*---------------------------------- EOF -------------------------------------*/
