/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	help.c - help screen
 *      --------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:04:02 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

#define	MAXHELPL	20

/*---------------------------------------------------------------------------*
 *	help function keys
 *---------------------------------------------------------------------------*/

static struct fk_tab help_keys[] =
{
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "},
	{"                ", "         "}
};

static char *helpstr[MAXHELPL] =
{
"   ----------------------- Command Window Commands: -------------------------",
"   Backspace - Delete left character    Delete    - Delete cursor character",
"   Control-F - Character forward        Control-B - Character backward",
"   Control-A - Begin of line            Control-E - End of line",
"   Control-N - Next line                Control-P - Previous line",
"   Control-K - Kill line from cursor    Control-Y - Yank back killed line",
"   Control-D - Exit back to system      \"%\" is the marked files wildcard char",
"   ESC B     - Word Backward            ESC F     - Word Forward",
"   ------------------------- File Window Commands: --------------------------",
"   Cursor up, Cursor down, Cursor right, Cursor left - Select a file",
"   Page Up   - Previous file page       Page Down - Next file page",
"   F1        - Edit current file        F2        - Echo name to commandline",
"   F3        - Display this help        F4        - Enter configuration menu",
"   F5        - Mark the current file    F6        - Unmark all marked files",
"   F7        - Alternate attribute line F8        - Close current directory",
"",
"   Alternate Invocation of F1...F8 by pressing ESCAPE and then '1'...'8'",
"   Alternate Invocation of Page up/down by pressing ESCAPE and then 'N' or 'P'",
};

char *contin = {"Press any key to continue ..."};

/*---------------------------------------------------------------------------*
 *	help page
 *---------------------------------------------------------------------------*/
void
help(void)
{
	int i;

	suspend_time();				/* stop updating time */

	cur_fktab = &help_keys;			/* help screen fkey label table */
	init_flabels();

	wmove(stdscr, 0, 0);			/* first line */
	wclear(stdscr);				/* clear window */
	touchwin(stdscr);
	wnoutrefresh(stdscr);

	move(0, 0);				/* first line */
	attron(A_REVERSE);			/* highlight on */
	addstr(headerline);			/* program/copyright header */
	attroff(A_REVERSE);			/* highlight off */

	for (i = 0; i < MAXHELPL; i++)
		mvaddstr(2 + i, 0, helpstr[i]);	/* print help screen */

	move(LINES - 1, 0);			/* last line */

	attron(A_REVERSE);			/* highlight on */
	addstr(contin);
	attroff(A_REVERSE);			/* highlight on */
	move(LINES - 1, strlen(contin));	/* last line */
	refresh();				/* show it */

	getch();				/* get any char from user */

	wmove(stdscr, 0, 0);			/* first line */
	wclear(stdscr);				/* clear window */
	touchwin(stdscr);
	wnoutrefresh(stdscr);

	touchwin(cmnd_w);
	touchwin(fst_w);
	touchwin(file_w);
	if (opt_attrib)
		touchwin(attr_w);
	if (opt_labels)
		touchwin(flbl_w);

	resume_time();				/* restart updating time */

	header();				/* new header */
	dis_hist();				/* display current commandline */
	fresh_files();				/* refresh files */
	attribs(1);				/* new attributes */
	update_all();				/* update complete screen */

	cur_fktab = &sys_keys;			/* system fkey label table */
	init_flabels();
}

/*----------------------- E O F -------------------------------------------*/
