/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	movement.c - file window "cursor" movement
 *      ------------------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:04:10 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

/*---------------------------------------------------------------------------*
 *	return true if filename is on current displayed page
 *---------------------------------------------------------------------------*/
static int
iscurpag(void)
{
	return (((cur_file->page) & PAGE_MSK) == cur_page);
}

/*---------------------------------------------------------------------------*
 *	move current file to first file on page
 *---------------------------------------------------------------------------*/
int
firstfile(void)
{
	cur_file = scr_beg;

	for (;;)
	{
		if (iscurpag())
			return (GOOD);
		else if (cur_file->next == NULL)
			return (BAD);
		else
			cur_file = cur_file->next;
	}
}

/*---------------------------------------------------------------------------*
 *	move current file to last file on page
 *---------------------------------------------------------------------------*/
int
lastfile(void)
{
	cur_file = scr_beg;			/* start of screen */

	for (;;)
	{
		if (cur_file->next == NULL)
			break;
		if ((cur_file->next->page) == ((cur_page + 1) | FIRST))
			break;
		cur_file = cur_file->next;
	}
	for (;;)
	{
		if (iscurpag())
			return (GOOD);
		else if (cur_file->prev == NULL)
			return (BAD);
		else
			cur_file = cur_file->prev;
	}
}

/*---------------------------------------------------------------------------*
 *	move current file to next file on page
 *---------------------------------------------------------------------------*/
int
nextfile(void)
{
	for (;;)
	{
		if (cur_file->next == NULL)
			return (BAD);
		cur_file = cur_file->next;
		if (iscurpag())
			return (GOOD);
	}
}

/*---------------------------------------------------------------------------*
 *	move current file to previous file on page
 *---------------------------------------------------------------------------*/
int
prevfile(void)
{
	for (;;)
	{
		if (cur_file->prev == NULL)
			return (BAD);
		cur_file = cur_file->prev;
		if (iscurpag())
			return (GOOD);
	}
}

/*---------------------------------------------------------------------------*
 *	move highlighted file to first file on page
 *---------------------------------------------------------------------------*/
void
move_home(void)
{
	cur_norm(cur_file);
	firstfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file to last file on page
 *---------------------------------------------------------------------------*/
void
move_hmdn(void)
{
	cur_norm(cur_file);
	lastfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos right, wrapping on end
 *---------------------------------------------------------------------------*/
void
move_right(void)
{
	cur_norm(cur_file);
	if (nextfile() == BAD)
		firstfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos left, wrapping on begin
 *---------------------------------------------------------------------------*/
void
move_left(void)
{
	cur_norm(cur_file);
	if (prevfile() == BAD)
		lastfile();
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos up, wrapping on top
 *---------------------------------------------------------------------------*/
void
move_up(void)
{
	struct onefile *save_cur;
	int upcol;

	save_cur = cur_file;			/* fail safe */

	upcol = cur_file->ocol;			/* current column */

	if (first != NULL)
	{
		cur_norm(cur_file);

		while (prevfile() != BAD)
		{
			if (cur_file->ocol == upcol)
				goto upfound;
		}
		if (lastfile() != BAD)
		{
			do
			{
				if (cur_file->ocol == upcol)
					goto upfound;
			}
			while (prevfile() != BAD);
		}
	}
	cur_file = save_cur;			/* not found, no change ! */

upfound:
	cur_inv(cur_file);
}

/*---------------------------------------------------------------------------*
 *	move highlighted file one pos down, wrapping on bottom
 *---------------------------------------------------------------------------*/
void
move_down(void)
{
	struct onefile *save_cur;
	int downcol;

	save_cur = cur_file;			/* fail safe */

	downcol = cur_file->ocol;		/* current column */

	if (first != NULL)
	{
		cur_norm(cur_file);

		while (nextfile() != BAD)
		{
			if (cur_file->ocol == downcol)
				goto downfound;
		}
		if (firstfile() != BAD)
		{
			do
			{
				if (cur_file->ocol == downcol)
					goto downfound;
			}
			while (nextfile() != BAD);
		}
	}
	cur_file = save_cur;			/* not found, no change ! */

downfound:
	cur_inv(cur_file);
}

/*---------------------------------- EOF -------------------------------------*/
