/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	rcinit.c - run-time initialization 
 *      ----------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:04:18 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"				/* everything we want */
#include "control.h"				/* control-characters */
#include "rcinit.h"				/* .cwishrc definitions */

static void onoff(char *string, int *var);
static void optchar(char *string, unsigned char *var);
static void optstr(char *string, char dest[]);

/*---------------------------------------------------------------------------*
 *	get user configuration from file "$HOME/.cwishrc"
 *---------------------------------------------------------------------------*/
void
readrc(void)
{
	char buffer[MAXPATHLEN + 1];		/* gp buffer */
	FILE *fp;

	sprintf(buffer, "%s/%s", envhome, CWISHRC);

	if ((fp = fopen(buffer, "r")) == NULL)
		return;

	while ((fgets(buffer, MAXPATHLEN, fp)) != NULL)
	{

/* comments */
		if (buffer[0] == '#' || buffer[0] == ' ' || buffer[0] == '\t')
		{
			continue;
		}
/* booleans */
		else if (!strncmp(buffer, RC_ATTR, strlen(RC_ATTR)))
		{
			onoff(buffer, &opt_attrib);
		}
		else if (!strncmp(buffer, RC_PRESERVE, strlen(RC_PRESERVE)))
		{
			onoff(buffer, &opt_preserve);
		}
		else if (!strncmp(buffer, RC_FKEYS, strlen(RC_FKEYS)))
		{
			onoff(buffer, &opt_labels);
		}
		else if (!strncmp(buffer, RC_LINKS, strlen(RC_LINKS)))
		{
			onoff(buffer, &opt_links);
		}
		else if (!strncmp(buffer, RC_POINT, strlen(RC_POINT)))
		{
			onoff(buffer, &opt_point);
		}
		else if (!strncmp(buffer, RC_RETURN, strlen(RC_RETURN)))
		{
			onoff(buffer, &opt_return);
		}
		else if (!strncmp(buffer, RC_WILDON, strlen(RC_WILDON)))
		{
			onoff(buffer, &opt_wildon);
		}
		else if (!strncmp(buffer, RC_CDROM, strlen(RC_CDROM)))
		{
			onoff(buffer, &opt_cdrom);
		}
		else if (!strncmp(buffer, RC_CDNOPROG, strlen(RC_CDNOPROG)))
		{
			onoff(buffer, &opt_cdnoprog);
		}
		else if (!strncmp(buffer, RC_DOTNAMES, strlen(RC_DOTNAMES)))
		{
			onoff(buffer, &opt_dotnames);
		}
		else if (!strncmp(buffer, RC_TAGMOVE, strlen(RC_TAGMOVE)))
		{
			onoff(buffer, &opt_tagmove);
		}
		else if (!strncmp(buffer, RC_FOLLOWSDIR, strlen(RC_FOLLOWSDIR)))
		{
			onoff(buffer, &opt_followsdir);
		}
		else if (!strncmp(buffer, RC_AUTOEXPAND, strlen(RC_AUTOEXPAND)))
		{
			onoff(buffer, &opt_autoexpand);
		}
/* strings */
		else if (!strncmp(buffer, RC_PAGER, strlen(RC_PAGER)))
		{
			optstr(buffer, opt_more);
		}
		else if (!strncmp(buffer, RC_EDITOR, strlen(RC_EDITOR)))
		{
			optstr(buffer, opt_edit);
		}
		else if (!strncmp(buffer, RC_WILDCARD, strlen(RC_WILDCARD)))
		{
			optstr(buffer, opt_wild);
		}
		else if (!strncmp(buffer, RC_TAIL, strlen(RC_TAIL)))
		{
			tail_cmd(buffer);
		}
		else if (!strncmp(buffer, RC_ALIAS, strlen(RC_ALIAS)))
		{
			alias_cmd(buffer);
		}

#ifdef USE_OWN_SYSTEM
		else if (!strncmp(buffer, RC_SHELL, strlen(RC_SHELL)))
		{
			optstr(buffer, opt_shell);
		}
#endif

/* characters */
		else if (!strncmp(buffer, RC_DELIMITER, strlen(RC_DELIMITER)))
		{
			optchar(buffer, &opt_delimiter);
		}
	}
}

/*---------------------------------------------------------------------------*
 *	process on/off strings
 *---------------------------------------------------------------------------*/
static void
onoff(char *string, int *var)
{
	char *ptr;

	if ((ptr = (char *) index(string, '=')) == NULL)
		return;

	ptr++;

	if (!strncmp(ptr, RC_ON, strlen(RC_ON)))
	{
		*var = 1;
	}
	else if (!strncmp(ptr, RC_OFF, strlen(RC_OFF)))
	{
		*var = 0;
	}
}

/*---------------------------------------------------------------------------*
 *	process chars
 *---------------------------------------------------------------------------*/
static void
optchar(char *string, unsigned char *var)
{
	unsigned char *ptr;

	if ((ptr = (unsigned char *) index(string, '=')) == NULL)
		return;

	ptr++;

	if (isprint(*ptr))
		*var = *ptr;
}

/*---------------------------------------------------------------------------*
 *	process strings
 *---------------------------------------------------------------------------*/
static void
optstr(char *string, char dest[])
{
	char *ptr, *bptr;

	if ((ptr = (char *) index(string, '=')) == NULL)
		return;

	ptr++;

	bptr = dest;

	while (*ptr && (*ptr != '#') && (isprint(*ptr)) && (!isspace(*ptr)))
		*bptr++ = *ptr++;

	*bptr = '\0';
}

/*---------------------------------------------------------------------------*
 *	tail_cmd is called while reading .cwishrc and is used to put
 *	parameters of a 'tail' keyword into the endings table etab[][].
 *---------------------------------------------------------------------------*/
void
tail_cmd(char *buffer)
{
	int i;
	char *pattern, *cmd;

	/* proceed to next free entry */

	for (i = 0; i < (sizeof(etab) / sizeof(etab[0])) && etab[i][0]; ++i)
		;

	if (i == (sizeof(etab) / sizeof(etab[0])))	/* space left ? */
		return;

	if ((pattern = index(buffer, '=')) == NULL)	/* syntax ok ? */
		return;

	pattern++;

	for (cmd = pattern; *cmd && !isspace(*cmd); ++cmd)	/* this is the tail */
		;

	if (*cmd)
		*cmd++ = 0;			/* terminate tail */

	for (; *cmd && isspace(*cmd); ++cmd)	/* skip delimiting whitespace */
		;


	for (buffer = cmd; *buffer && isprint(*buffer); ++buffer)	/* command */
		;

	*buffer = 0;				/* delimit command */

	if (*pattern && *cmd)			/* syntax ok ? */
	{
		etab[i][0] = strdup(pattern);	/* put the tail pattern and */
		etab[i][1] = strdup(cmd);	/*  the command into table  */
	}
}

/*---------------------------------------------------------------------------*
 *	alias_cmd is called while reading .cwishrc and is used to put
 *	parameters of an 'alias' keyword into the alias table atab[][].
 *---------------------------------------------------------------------------*/
void
alias_cmd(char *buffer)
{
	int i;
	char *pattern, *cmd;

	for (i = 0; i < (sizeof(atab) / sizeof(atab[0])) && atab[i][0]; ++i)
		;

	if (i == (sizeof(atab) / sizeof(atab[0])))
		return;

	if ((pattern = index(buffer, '=')) == NULL)
		return;

	pattern++;

	for (cmd = pattern; *cmd && !isspace(*cmd); ++cmd)
		;

	if (*cmd)
		*cmd++ = 0;

	for (; *cmd && isspace(*cmd); ++cmd)
		;


	for (buffer = cmd; *buffer && isprint(*buffer); ++buffer)
		;

	*buffer = 0;

	if (*pattern && *cmd)
	{
		atab[i][0] = strdup(pattern);
		atab[i][1] = strdup(cmd);
	}
}

/*--------------------------------- EOF ------------------------------------*/
