/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	showtime.c - display time in header line
 *      ----------------------------------------
 *
 *	Last Edit-Date: [Sat Jan  1 17:19:12 2000]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

void timeout_hdlr(int sig);
static void make_time(void);
static int check_mail(void);

#define ALARMTIME	15

/*---------------------------------------------------------------------------*
 *	init time display in header
 *---------------------------------------------------------------------------*/
void
init_time(void)
{
	signal(SIGALRM, &timeout_hdlr);
	make_time();
	alarm(ALARMTIME);
}

/*---------------------------------------------------------------------------*
 *	print current time into header
 *---------------------------------------------------------------------------*/
static void
make_time(void)
{
	static char *wd[] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
	
	struct tm *tmptr;
	time_t t;
	char buf[80];

	time(&t);
	tmptr = localtime(&t);

	if(check_mail())
		strcpy(headerline + MAIL_POS, "mail                      ");
	else
		strcpy(headerline + MAIL_POS, "                          ");	
	
#ifdef DATE_US
	sprintf(buf, "%s %02d/%02d/%02d %02d:%02d",
		wd[tmptr->tm_wday], tmptr->tm_mon+1,
		tmptr->tm_mday,
		YEAR(tmptr->tm_year),
		tmptr->tm_hour, tmptr->tm_min);
#else
	sprintf(buf, "%s %02d.%02d.%02d %02d:%02d",
		wd[tmptr->tm_wday], tmptr->tm_mday,
		tmptr->tm_mon+1,
		YEAR(tmptr->tm_year),
		tmptr->tm_hour, tmptr->tm_min);
#endif
	strcpy(headerline + TIME_POS, buf);
}

/*---------------------------------------------------------------------------*
 *	the timeout handler
 *---------------------------------------------------------------------------*/
void
timeout_hdlr(int sig)
{
	int i;

	signal(SIGALRM, SIG_IGN);

	make_time();

	if (!errorflag)
	{
		i = 0;
		if (COLS > 80)
			i = (COLS - 80) / 2;
		wmove(cmnd_w, C_HEAD, MAIL_POS + i);
		wattron(cmnd_w, A_REVERSE);
		waddstr(cmnd_w, headerline + MAIL_POS);
		wattroff(cmnd_w, A_REVERSE);
		wmove(cmnd_w, C_LINE, curcol());
		wrefresh(cmnd_w);
	}
	alarm(ALARMTIME);
	signal(SIGALRM, timeout_hdlr);
}

/*---------------------------------------------------------------------------*
 *	stop updating the time
 *---------------------------------------------------------------------------*/
void
suspend_time(void)
{
	signal(SIGALRM, SIG_IGN);
}

/*---------------------------------------------------------------------------*
 *	restart updating the time
 *---------------------------------------------------------------------------*/
void
resume_time(void)
{
	signal(SIGALRM, &timeout_hdlr);
	make_time();
}

/*---------------------------------------------------------------------------*
 *	check for new mail
 *---------------------------------------------------------------------------*/
static int
check_mail(void)
{
	struct stat sb;

	if(envmail)
	{	
		if((stat(envmail, &sb)) != 0)
			return(0);
		if(sb.st_atime < sb.st_mtime)
			return(1);
	}
	return(0);
}
/*---------------------------------- EOF -------------------------------------*/
