/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	system.c - configurable system() call
 *      -------------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:04:30 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

#ifdef USE_OWN_SYSTEM

int
mysystem(const char *cmdstring)
{
	pid_t pid;
	void *intsave, *quitsave;
	int omask;
	int pstat;
	static char *pname;

	if (!cmdstring)				/* just checking... */
		return (1);

	if ((pname = rindex(opt_shell, '/')) == NULL)
		pname = opt_shell;
	else
		pname++;

	omask = sigblock(sigmask(SIGCHLD));

#ifdef USE_VFORK
	switch (pid = vfork())
#else
	switch (pid = fork())
#endif
	{
		case 0:			/* child */
			(void) sigsetmask(omask);
			execl(opt_shell, pname, "-c", cmdstring, (char *) NULL);
			_exit(127);

		case -1:			/* error */
			(void) sigsetmask(omask);
			return (-1);
	}

	intsave = signal(SIGINT, SIG_IGN);
	quitsave = signal(SIGQUIT, SIG_IGN);

	pid = waitpid(pid, (int *) &pstat, 0);

	(void) sigsetmask(omask);

	(void) signal(SIGINT, intsave);
	(void) signal(SIGQUIT, quitsave);

	return (pid == -1 ? -1 : pstat);
}

#endif /* USE_OWN_SYSTEM */

/*--------------------------------- EOF -------------------------------------*/
