/*
 *
 * Copyright (c) 1988, 1999 Hellmuth Michaelis. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 *
 *	usrgrp.c - user/group display routines
 *      --------------------------------------
 *
 *	Last Edit-Date: [Fri Jul 30 11:04:42 1999]
 *
 *	$Id$
 *
 *----------------------------------------------------------------------------*/

#ifndef HAS_USR_GRP

#include "cwish.h"

#ifndef min
#define min(x, y)	(((x) < (y)) ? (x) : (y))
#endif

#define	NCACHE		256
#define	MASK		(NCACHE - 1)
#define NAMELENGTH	16

/*---------------------------------------------------------------------------*
 *	make a user id string from the numeric user id (and cache a bit)
 *---------------------------------------------------------------------------*/
char *
user_from_uid(uid_t uid, int size)
{
	static struct ncache
	{
		uid_t uid;
		char name[NAMELENGTH];
	}
	c_uid[NCACHE];

	static char nbuf[15];
	register struct passwd *pw;
	register struct ncache *cp;
	int length = min(NAMELENGTH, size);

	cp = (c_uid + (uid & MASK));

	if (cp->uid != uid || !*cp->name)
	{
		if (!(pw = getpwuid(uid)))
		{
			sprintf(nbuf, "%-*u", length, (unsigned int)uid);
			return (nbuf);
		}
		cp->uid = uid;
		sprintf(cp->name, "%-*s", NAMELENGTH, pw->pw_name);
	}
	strncpy(nbuf, cp->name, length);
	return (nbuf);
}

/*---------------------------------------------------------------------------*
 *	make a group id string from the numeric group id (and cache a bit)
 *---------------------------------------------------------------------------*/
char *
group_from_gid(gid_t gid, int size)
{
	static struct ncache
	{
		gid_t gid;
		char name[NAMELENGTH];
	}
	c_gid[NCACHE];

	static char nbuf[15];
	register struct group *gr;
	register struct ncache *cp;
	int length = min(NAMELENGTH, size);

	cp = c_gid + (gid & MASK);
	if (cp->gid != gid || !*cp->name)
	{
		if (!(gr = getgrgid(gid)))
		{
			sprintf(nbuf, "%-*u", length, (unsigned int)gid);
			return (nbuf);
		}
		cp->gid = gid;
		sprintf(cp->name, "%-*s", NAMELENGTH, gr->gr_name);
	}

	strncpy(nbuf, cp->name, length);
	return (nbuf);
}

#endif

/*---------------------------------- EOF -------------------------------------*/
